    %Ginters Buss 2012
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % if useful, cite Ginters Buss "Real-time signal extraction with
    % regularized multivariate direct filter approach",
    % in Journal of Forecasting
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%-----------------------------------------------------------------
% 1. READING THE DATA AND PERFORMING TRANSFORMATIONS
% Read the data: please adapt the path to your setting
% I'm working with scaled log levels (no differences)
clc;
clear;
close all;

%set default graphics 
set(0,'DefaultAxesColorOrder',[0 0 0;1 0 0;0 0 1],...
      'DefaultAxesLineStyleOrder','-|--.|-.|--|:|-+|-*|-s|-o|-v|-^',...
      'DefaultLineMarkerSize',3);

addpath(genpath('myfunc')) 

%if use garch:
%addpath(genpath('C:\Users\gintersb\data\matlab\external codes\mtlb_tlbx')) % if don't have optim tbx
%addpath(genpath('C:\Users\gintersb\data\matlab\external codes\Sheppard financial\MFEToolbox')) % garch

%addpath(genpath('C:\Users\gintersb\data\latex\regularized_mdfa\m-code\realtime_evaluation')) %real-time functions

% some explanatory vars from eurostat (industrial production, employment expectations, order-book assessmets, and stocks of finished goods)

%%% data params
sa_data=1; %choose sa data(1) or nsa data(0) or x12 satble sa (2)
sa_gdp=1; % choose sa(1) or nsa(0) gdp as target
gap=3; % # months cut off from the end of gdp
qoq=0; %mom (0) or qoq(1) transform

regdiffexplanvar=1; %  diff(1) or no diff(0) ecfin data
yoygdp=0; % yoy transform

evalspan=0:10*12; 
updateduringnegativeg=1; % 1 - update during negative growth
updatefreq=2; %2 (frequent) or 12 (not that frequent)
logquery=0; % 1 - log; 0 no log GDP
regdiff=0; % 1 - regular diff; 0 - no regular diff GDP
usesmallsetofvars=0; % use small set of vars

%germany=1; % 9 vars
%france=1;  % 9 vars 
%italy=1;   % 8 vars
%spain=1;   % 9 vars


%%%% filter params
squeeze=1; % if =1 squeezes the last bit of information; 0-no
garch=0; % garch(1) or no garch(0) of explan vars
numbseries=100;  %how many explan series at a time, in filter; inf or a big # if all
maxfilterlength=12;
minfilterlength=12;
rtio=1.1; %1.1 %1.2-3.6
pp=1; %power for MSFE weights
expweight=0; %.5 %1.9
lin_expweight=0;
Lag=0; %-7
lambda=0;  %15
usefactors=0; % use princomp or original data
    nfactors=3; % # factors; only if usefactors=1
    ftype='outer'; % 'outer', 'corr' or ' cov'    
filterdepvar=1; % filter dep var or not; doesn't seem to help
nostand=0; %1: do not scale the output by the variance of finite symm
standtarget=0; %1: standardize the target variable before input in filter; not welcome;
i1=0;   %Do not impose i1<-T for bandpass or highpass filters;necessary for low-pass filters applied to integrated time series (otherwise signal and real-time estimates are not cointegrated i.e. the filter error is not stationary). Do not impose the restriction in the case of bandpass filters (unless you are willing to endure inefficiency)
i2=0;   % do not impose for true bandpass; i1<T and i2<-T are both necessary in the case of I(2) processes with a double unit-root in frequency zero (I don't know any application requiring an I(2) specification, though; My(Wildi) personal preference is i1<-F and i2<-T which cannot be realized by model-based approaches. 

weightbycorr=0; % weight amplitudes at zero freq for explan vars  by corr with dep var                          

lambda_decay=.2; %.1
lambda_cross=.2; %.01
lambda_smooth=0; %0.000000000001;

playground=1; % 9/15 playground=[0,1] unity means lowpass if i1=1
freqdom=0;  %0 - linearly interpolate gdp; 1 - frequency domain solution

% plot(b)
% plot(abs(trf))

ord=9e5; %9e5 (ideal, fast)  %length of target/ideal filter; 
ub=6;  %the same as 2pi/18
lb=Inf;  %48(bandpass) or inf(lowpass); %


if sa_data==1
    [edata1_data,edata1_string]=xlsread('..\lv_data_sa.xls');
elseif sa_data==0
    [edata1_data,edata1_string]=xlsread('C:\Users\gintersb\data\database\ea_data_sa_firstrelease.xls');
elseif sa_data==2
    [edata1_data,edata1_string]=xlsread('C:\Users\gintersb\data\database\ea_data_sa_stable_x12.xls');
end

edata1_names=edata1_string(1,:);
edata1_dates=edata1_string(:,1);
if sa_gdp==0
[gdpdta_data,gdpdta_string]=xlsread('C:\Users\gintersb\data\latex\regularized_mdfa\m-code\ea_gdp.xls');
elseif sa_gdp==1
[gdpdta_data,gdpdta_string]=xlsread('..\lv_gdp_sa.xls');    
end
gdpdta_names=gdpdta_string(1,:);
gdpdta_dates=gdpdta_string(:,1);
gdp1=gdpdta_data;

gdpsize=size(gdp1);

gdpint=zeros(3*(gdpsize(1)-1)+1,1);
gdpint(1)=gdp1(1);
for i=1:gdpsize(1)-1
    gdpint(3*i+1)=gdp1(i+1);
end
for i=1:gdpsize(1)-1
    gdpint(3*i-1)=gdpint(3*i-2)+(gdpint(3*i+1)-gdpint(3*i-2))/3;
    gdpint(3*i)=gdpint(3*i-1)+(gdpint(3*i+1)-gdpint(3*i-2))/3;
end
    
size(gdpint)  

if qoq==0 
    if logquery==1 
        gdpint1=log(gdpint);
        dgdpint=100*(gdpint1(2:end)-gdpint1(1:end-1));  % seas diff
    else
        if yoygdp % yoy transform of dep var
            dgdpint=100*(gdpint(13:end)./gdpint(1:end-12) - 1);
        else
            dgdpint=100*(gdpint(2:end)./gdpint(1:end-1) - 1);  % seas diff, percentage
        end
    end
elseif qoq==1
    if logquery==1 
        gdpint1=log(gdpint);
        dgdpint=100*(gdpint1(4:end)-gdpint1(1:end-3));  % seas diff
    else
        dgdpint=100*(gdpint(4:end)./gdpint(1:end-3) - 1);  % seas diff, percentage
    end
end

if regdiff==1
    dgdpint=dgdpint(2:end) - dgdpint(1:end-1);  % regular diff
end


%%%%%%%%%% explanatory data %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  
edata=edata1_data; 

edsize=size(edata(:,:));
edata1=edata(:,:);
%%%%%%%%%% reverse the signs of some vars %%%%%%%%%%%%%%%%%
%{
%%
aaa=50;
signn=1;
plot([(edata1(:,1)-nanmean(edata1(:,1)))/nanstd(edata1(:,1)),(signn*edata1(:,aaa)-nanmean(signn*edata1(:,aaa)))/nanstd(signn*edata1(:,aaa))])
%%
%}

%%%%%%% log some of the data that are trending %%%%%%%%%%%%%%%%%%%%%%%%
%
%aaa=50;
%signn=1;
%plot([(edata1(:,1)-nanmean(edata1(:,1)))/nanstd(edata1(:,1)),(signn*edata1(:,aaa)-nanmean(signn*edata1(:,aaa)))/nanstd(signn*edata1(:,aaa))])
%

edata1(:,4)=-edata1(:,4);
edata1(:,34)=-edata1(:,34);
edata1(:,35)=-edata1(:,35); % job vacancies

logwhich=[31,32,33,36,37,38,39,40];

edata1(:,logwhich)=log(edata1(:,logwhich));

if regdiffexplanvar
    edata1=edata1(2:end,:) - edata1(1:end-1,:);
else
    edata1(2:end,logwhich)=edata1(2:end,logwhich)-edata1(1:end-1,logwhich);
    edata1(1,logwhich)=nan;
end


alignends=1; % 0 - no align; drop missing at ends; 1 - align with lags;
k = find(isnan(edata1(end,:)));
if alignends==1
    while ~isempty(k)
        edata1(:,k)=[k*NaN;edata1(1:end-1,k)];
        k = find(isnan(edata1(end,:)));
    end
else
    while ~isempty(k)
        edata1(end,:)=[];
        k = find(isnan(edata1(end,:)));
    end
end     



excessobs=size(edata1,1)-size(dgdpint,1)-4;

edat=edata1(excessobs+1:end,:);

%%%%% exclude some crappy vars %%%%%%%%%%%%%%

excludevars=[]; %[1,8,19,22,25,35];
edat(:,excludevars)=[];

if usesmallsetofvars
    edat=edat(:,[9,2,4,5,7,15,1,8]);
end

%edatt=edat;
%edatt(:,1)=edat(:,10);
%edatt(:,10)=edat(:,1);
%edat=edatt;
%plot(abs(trf))

%  plot(b)
% plot(abs(trf))

edat(1:12,:)=[];
dgdpint(1:12)=[];
% 
% for i=1:size(edat,2)
%     edat(:,i)=(edat(:,i)-mean(edat(:,i)))/std(edat(:,i));
% end
% 
% 
% if usefactors
% [weights,princomp,eigenvals,explvar,cumr2]=pca(edat,ftype); %type: 'outer', 'cov' or 'corr'
% 
% edat=princomp(:,1:nfactors);
% end

dgdpintsize=size(dgdpint);        
edatsize=size(edat);


%%%%%% real-time evaluation

realtimeres=nan(edatsize(1),1);

reveragavvector = nan(evalspan(end)+1,1);
aicagavvector = nan(evalspan(end)+1,1);
dgravvector = nan(evalspan(end)+1,2);


for jjj=evalspan
    dgdpint1=dgdpint(1:end-gap-jjj);
    edat1=edat(1:end-jjj,:);
    
    [b,trf,vari,reveragav,aicagav,dgrav,weight_constraint]=rmdfa_fn(dgdpint1,edat1,squeeze,garch,numbseries,maxfilterlength,minfilterlength,rtio,pp,expweight,...
  Lag,  lambda,usefactors,nfactors,ftype,filterdepvar,nostand,standtarget,i1,i2, ...
  weightbycorr,lambda_decay,lambda_cross,lambda_smooth,playground,freqdom,...
  ord, ub,lb,lin_expweight);
    realtimeres(end-jjj)=vari(end);
    reveragavvector(end-jjj)=reveragav;
    aicagavvector(end-jjj)=aicagav;
    dgravvector(end-jjj,:)=dgrav;
end

mean_MSFE=mean(reveragavvector)
mean_aic=mean(aicagavvector)
mean_dgr=mean(dgravvector,1)





%%%%%%%% which series are off %%%%%%%%%%%%%%%%%%%%
%{
%%
aaa=50;
signn=1;
plot((dgdpint(:,1)-nanmean(dgdpint(:,1)))/nanstd(dgdpint(:,1)))
hold on
plot((signn*edat(:,aaa)-nanmean(signn*edat(:,aaa)))/nanstd(signn*edat(:,aaa)))
hold off
%%
%}


%{
%##### test for normality ##########
%kolmogorov
[stat,pval,H] = kolmogorov(edat(:,8),[],'normcdf',0,1)
% neither rejects the null of normality
%Jarque-Bera
[statistic, pval,H] = jarquebera(edat(:,8)) % 5 fails
% another JB
H=jbtest(edat(:,8))
%}
% 
% if garch==1
%     options  =  optimset('fminunc');
%     options.Display =  'off';
% 
%     for i=1:size(edat,2)
%         [a1,a2,a3,a4,a5,a6,a7]=tarch(edat(:,i),1,0,1,[],[],[],options);
%         edat(:,i)=edat(:,i)./sqrt(a3);   
%     end
%  
%     
% for i=1:size(edat,2)
%     edat(:,i)=(edat(:,i)-mean(edat(:,i)))/std(edat(:,i));
% end    
% end

%plot data
%{
close all

figure('Position',[300 200 800 450])

plot([nan(11,1);(dgdpint-mean(dgdpint))/std(dgdpint)],'black','LineWidth',3)
hold on
plot(edat,'black','MarkerSize',3)
hold off
legend('yoy gdp','prod.ind','order.book.ind','stocks.ind','prod.expect.ind','empl.expect.ind','build.conf','retail.conf','consumers.conf','service.conf','Location','SouthWest')
    
datevar=edata1_dates(nsanumber+1:end,:);
sizedatevar=size(datevar);
for i=1:sizedatevar(1)
    tempdate=datevar{i};
    datevar{i}=tempdate(4:end);
end
nticks=fix((edatsize(1)+24)/24);
%title(strcat('Euro-light: monthly zero-lag real-time indicator, last obs: ',datevar(end)))
%ylabel('q-o-q GDP(green), symm.filt(red), q-o-q ind(blue), q-o-q Ecoin(black)')
axis([-Inf edatsize(1)+2 -Inf +Inf])
set(gca,'XTick',edatsize(1)+1-nticks*24+(1:nticks)*24)
if regdiffexplanvar==0
set(gca,'XTickLabel',[datevar(edatsize(1)-nticks*24+(1:nticks)*24)])
elseif regdiffexplanvar==1
set(gca,'XTickLabel',[datevar(edatsize(1)+1-nticks*24+(1:nticks)*24)])    
end
    
print(1,'-depsc','C:\Users\gintersb\data\latex\regularized_mdfa\data_yoy')
print(1,'-dpng','C:\Users\gintersb\data\latex\regularized_mdfa\data_yoy')

%lv
%legend('IKP gada pieaugumi','prod. rpn.','pastjumi rpn.','krjumi rpn.','raoanas gaidas rpn.','nodarbintbas gaidas rpn.','bvniecbas konf.','mazumtirdzniecbas konf.','patrtju konf.','pakalpojumu konf.','Location','SouthWest')
    
%print(1,'-depsc','C:\Users\gintersb\data\latex\regularized_mdfa\data_yoy')
%print(1,'-dpng','C:\Users\gintersb\data\latex\regularized_mdfa\data_yoy')

%}

%plot gdp vs cross-sectional mean and the first pc of expl data
%{
[weights,princomp,eigenvals,explvar,cumr2]=pca(edat,ftype); %ftype=outer,cov,corr
close all
set(0,'DefaultAxesColorOrder',[0 0 0;1 0 0;0 0 1],...
      'DefaultAxesLineStyleOrder','-|--.|-.|--|:|-+|-*|-s|-o|-v|-^')
figure('Position',[300 200 800 450])

plot([nan(11,1);(dgdpint-mean(dgdpint))/std(dgdpint)],'black','LineWidth',3)
hold on
plot([mean(edat,2),(princomp(:,1)-mean(princomp(:,1)))/std(princomp(:,1))],'black','MarkerSize',3)
hold off
legend('standardized yearly growth of gdp','cross-sectional mean of standardized expl. data','standardized first principal component of expl. data','Location','SouthWest')
    
datevar=edata1_dates(nsanumber+1:end,:);
sizedatevar=size(datevar);
for i=1:sizedatevar(1)
    tempdate=datevar{i};
    datevar{i}=tempdate(4:end);
end
nticks=fix((edatsize(1)+24)/24);
%title(strcat('Euro-light: monthly zero-lag real-time indicator, last obs: ',datevar(end)))
%ylabel('q-o-q GDP(green), symm.filt(red), q-o-q ind(blue), q-o-q Ecoin(black)')
axis([-Inf edatsize(1)+2 -Inf +Inf])
set(gca,'XTick',edatsize(1)+1-nticks*24+(1:nticks)*24)
if regdiffexplanvar==0
set(gca,'XTickLabel',[datevar(edatsize(1)-nticks*24+(1:nticks)*24)])
elseif regdiffexplanvar==1
set(gca,'XTickLabel',[datevar(edatsize(1)+1-nticks*24+(1:nticks)*24)])    
end
    
print(1,'-depsc','C:\Users\gintersb\data\latex\regularized_mdfa\mean_pca_yoy')
print(1,'-dpng','C:\Users\gintersb\data\latex\regularized_mdfa\mean_pca_yoy')

%lv
legend('standartizti IKP gada pieaugumi','standartiztu izskaidr.rdt. rsgriezuma vidjais','izskaidr.rdt. standartizta pirm galven komponente','Location','SouthWest')
   
print(1,'-depsc','C:\Users\gintersb\data\latex\regularized_mdfa\mean_pca_yoy_lv')
print(1,'-dpng','C:\Users\gintersb\data\latex\regularized_mdfa\mean_pca_yoy-lv')

hold on
plot((vari-nanmean(vari))/nanstd(vari),'blue')
plot([(dgdpint-nanmean(dgdpint))/nanstd(dgdpint)],'black','LineWidth',3)
hold off
%}


%iipeur=(edata(22:end-1,end)-mean(edata(22:end-1,end)))/stdev(edata(22:end-1,end))


%plot([edat(:,1),iipeur])
% extract the data: in first differences (emphasize growth) - see below
% Define Data matrix:
% First column: target variable iip (target is symmetrical bandpass applied to IP) - see below
% Second to last columns: explaining variables (iip is included again, but now it's an explaining variable)


%-------------------------------------------------
% 2. Plot the stuff


        
[lcoin_data,lcoin_string]=xlsread('..\lcoin.xlsx');
lcoin_yoy=100*(exp((lcoin_data(1+9:end)+lcoin_data(1+6:end-3)+lcoin_data(1+3:end-6)+lcoin_data(1:end-9))/100)-1);



%close all;        
startval=70;
vari=realtimeres;
sizevar=size(vari(startval:end,end))
sizedgdpint=size(dgdpint(startval:end))

figure('Position',[300 200 800 450])
    plot(vari(startval:end,end), 'k','LineWidth',2) %[nan(119,1);[combined]]
  hold on
    %plot((mean(edat(startval:end,:),2)-mean(mean(edat(startval:end,:),2)))/ ...
    %    std(mean(edat(startval:end,:),2))*std(3*dgdpint(startval:end))+...
    %    mean(3*dgdpint(startval:end)),'-.k','LineWidth',2)
    %plot(rt_mean(startval:end),'-.k','LineWidth',2)
   plot([nan(3,1);100*lcoin_data],'-.k','LineWidth',2)
        %plot(dgdp_finsym(1:end),'red')
        if sa_gdp==1 
        plot(3*dgdpint(startval:end),'k')
        elseif sa_gdp==0
        plot(3*sadgdpint(9+startval-1:end),'k')
        end
    
plot(zeros(sizevar(1),1),'k--')
datevar=edata1_dates(excessobs+12+startval:end,:);
sizedatevar=size(datevar);
for i=1:sizedatevar(1)
    tempdate=datevar{i};
    datevar{i}=tempdate(4:end);
end
nticks=fix((sizevar(1)+24)/24);
%title(strcat('EA-light: monthly real-time indicator, last obs: ',datevar(end)))
%ylabel('q-o-q EA GDP(green), symm.filt.(red), EA-light(blue)')
legend('filter output','Latcoin','quarterly growth of LV GDP,%','Location','SouthWest')
%legend('new indicator','yoy EA GDP','Eurocoin yoy','Location','SouthWest')
axis([-Inf sizevar(1)+2 -Inf +Inf])
set(gca,'XTick',sizevar(1)-nticks*24+(1:nticks)*24)
if regdiffexplanvar==0
set(gca,'XTickLabel',[datevar(sizevar(1)+2-nticks*24+(1:nticks)*24)])
elseif regdiffexplanvar==1
set(gca,'XTickLabel',[datevar(sizevar(1)+3-nticks*24+(1:nticks)*24)])    
end
hold off

%print(1,'-depsc','C:\Users\gintersb\data\latex\regularized_mdfa\9var_EAqoq_mean')
%print(1,'-depsc','C:\Users\gintersb\data\latex\regularized_mdfa\9var_EAqoq_rt_mean')
%print(1,'-depsc','C:\Users\gintersb\data\latex\regularized_mdfa\9var_EAqoq')
%print(1,'-depsc','C:\Users\gintersb\data\latex\regularized_mdfa\highdim_qoq_ea')
%print(1,'-depsc','C:\Users\gintersb\data\latex\regularized_mdfa\many_qoq_ea')
%print(1,'-depsc','C:\Users\gintersb\data\latex\regularized_mdfa\41var_lv_qoq')





