function [x_full,F]=em_principal_components(X,r,lag_X,Maximum_error)

% Reference: Stock & Watson 2002
%
%[x_full,F]=em_principal_components(X,r,lag_X,Maximum_error)
%defaults:
%r=2
%lag_X=1
%Max_error=0.001


% EM_PC Algorithm ---------------------------------------------------------

% Augmented Matrix with lead/lag structure -------------------


% Compile matrix with lead/lag structure
Nx=size(X,2);
x_enlarged=[];
for l=-lag_X:1:lag_X
    x_enlarged=[x_enlarged, [NaN(lag_X+l,Nx); X; NaN(lag_X-l,Nx)]];
end

% Replace missing values by 0
x_full=nansum(x_enlarged,3);

% Bai/Ng criteria

lambda=[];
for rr=1:10
    [IC(rr,1),lambda_full]=bai_ng(x_full,rr);
    lambda(rr,1)=lambda_full(rr,1);
end

[~,~,lambda_full]=principal_components(x_full,size(x_full,2));
lambda_sum=sum(lambda_full,1);
%{
fprintf('%18s %20s \n','Bai/Ng Criteria','Cumulative Lambda');
for rr=1:10
    if IC(rr,1)==min(IC)
        % s - string format, f - form feed, 11 - length from margin, .3 -
        % number of decimals, \n - new line
        fprintf('%11s %7.3f %11.3f \n',['IC(' num2str(rr) ')* ='],IC(rr,1),sum(lambda(1:rr,1),1)/lambda_sum);
    else
        fprintf('%10s %8.3f %11.3f \n',['IC(' num2str(rr) ') ='],IC(rr,1),sum(lambda(1:rr,1),1)/lambda_sum);
    end
end;
%}
% Main loop %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%disp(' ')
%disp('Running EM_PC algorithm with lag/lead structure:')

Change=1;
iteration=0;

while Change>Maximum_error
     iteration=iteration+1;

    % Factorization
    % PC - factors scores, U - eigenvec
    [F,U,~] = principal_components(x_full,r); 

    % Forecasting of x
    x_old=x_full;
    x_new=F*U'/((1+2*lag_X)*Nx);

    % Create auxiliary matrix, which values are used for x_full
    x_smpl=[x_new(1:2*lag_X,1:Nx);x_new(lag_X+1:end-lag_X,Nx*lag_X+1:Nx*lag_X+Nx);x_new(end-2*lag_X+1:end,end-Nx+1:end)];
        
    % Fill the initial matrix asymptotically with values from x_new
    x_enlarged_A=[];
    for l=-lag_X:1:lag_X
        x_enlarged_A=[x_enlarged_A, x_smpl(lag_X+1-l:-lag_X+end-l,:)];
    end

    x_full=nansum(x_enlarged,3)+(isnan(x_enlarged)==1).*x_enlarged_A;
    
    if iteration>1,
        Change=sum(sum((x_full-x_old).^2,1),2)/sum(sum(isnan(x_enlarged),1),2);
        %fprintf('  Error: %4.4f\n', Change)
    else
        %disp('  Error: NaN')
    end
end

x_full=x_full(:,1:Nx);


    




