function [des_mat,reg_mat,reg_xtxy,w_eight]=mat_func(i1,i2,L,weight_h_exp,lambda_decay,lambda_cross,lambda_smooth,Lag,weight_constraint)
    %Ginters Buss 2012
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % % if useful, cite Ginters Buss "Real-time signal extraction with
    % regularized multivariate direct filter approach",
    % in Journal of Forecasting
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % translated from Wildi I-mdfa_new6.r code 
    % New 2012-code: Computes regularization matrices and expresses 
    % parameters in central-deviance form
    % [des_mat,reg_mat,reg_xtxy,w_eight]=mat_func(i1,i2,L,weight_h_exp,
    % lambda_decay,lambda_cross,lambda_smooth,Lag,weight_constraint)
    % Lag cut suspended by GB 2012
    %if Lag>(L-1)/2
    %    disp('Lag larger than L/2!!!!! Will be trimmed automtically to L/2 (symmetric filter)');
    %    Lag=fix(L/2);
    %end
    lambda_smooth=abs(lambda_smooth);
    lambda_cross=abs(lambda_cross);
    lambda_decay=abs(lambda_decay);
    % The smoothness and decay regularization are conveniently (rightly) 
    % implemented on original parameters
    % The Q_smooth and Q_decay matrices address regularizations for original
    % unconstrained parameters (Therefore dimension L^2)
    % At the end, the matrix des_mat is used to map these regularizations 
    % to central-deviance parameters
    % accounting for first order constraints!!!!
    Q_smooth=zeros(L*length(weight_h_exp(1,:)),L*length(weight_h_exp(1,:)));
    Q_decay=Q_smooth;
    % Cross-sectional regularization if dimension>1
    if length(weight_h_exp(1,:))>1
        % The cross-sectional regularization is conveniently implemented 
        % on central-deviance parameters. The regularization is expressed on the
        % unconstrained central-deviance parameters (dimension L), then 
        % mapped to the original (unconstrained) parameters (dimension L) with Q_centraldev_original
        % and then mapped back to central-deviance with constraint (dim L-1) 
        % with des_mat (mathematically unnecessarily complicate but more convenient to implement in code).
        Q_cross=Q_decay;
        Q_centraldev_original=Q_cross;
    end
    for i=1:L
        % For symmetric filters or any historical filter with Lag>0 the 
        % decay must be symmetric about b_max(0,Lag) zunehmen
        Q_decay(i,i)=lambda_decay*(1+lambda_decay)^(2*abs(i-1-max(0,Lag)));
        if i==1
            Q_smooth(i,i:i+2)=lambda_smooth*[1,-2,1];
        elseif i==2
            Q_smooth(i,i-1:i+2)=lambda_smooth*[-2,5,-4,1];
        elseif i==L 
            Q_smooth(i,i-2:i)=lambda_smooth*[1,-2,1];
        elseif i==L-1
            Q_smooth(i,i-2:i+1)=lambda_smooth*[1,-4,5,-2];
        else
            Q_smooth(i,i-2:i+2)=lambda_smooth*[1,-4,6,-4,1];
        end
    end
    
    if length(weight_h_exp(1,:))>1
        for j=1:max(1,length(weight_h_exp(1,:))-1)   %j<-1
            Q_smooth(j*L+(1:L),j*L+(1:L))=Q_smooth(1:L,1:L);
            Q_decay(j*L+(1:L),j*L+(1:L))=Q_decay(1:L,1:L);
        end
        Q_centraldev_original(1:L,1:L)=diag(ones(1,L));
        Q_centraldev_original(1:L,L+(1:L))=(-1)*diag(ones(1,L));
        for i=2:length(weight_h_exp(1,:))
            Q_centraldev_original((i-1)*L+(1:L),1:L)=diag(ones(1,L));
            Q_centraldev_original((i-1)*L+(1:L),(i-1)*L+(1:L))=diag(ones(1,L));
            Q_centraldev_original(1:L,(i-1)*L+(1:L))=(-1)*diag(ones(1,L));
        end
        Q_centraldev_original=inv(Q_centraldev_original);
        Q_cross(L+(1:((length(weight_h_exp(1,:))-1)*L)),L+(1:((length(weight_h_exp(1,:))-1)*L)))=lambda_cross*diag(ones(1,((length(weight_h_exp(1,:))-1)*L)));
    end
    % weight vector (relevant if i1<-T)
    if i1
        if i2
            w_eight=[zeros(1,L-2),(L-1)*weight_constraint(1),-(L-2)*weight_constraint(1)];
            if length(weight_h_exp(1,:))>1
                for j = 2:length(weight_h_exp(1,:))
                    w_eight=[w_eight,zeros(1,L-2),(L-1)*weight_constraint(j),-(L-2)*weight_constraint(j)];
                end
            end
        else
            w_eight=[zeros(1,L-1),weight_constraint(1)];
            if length(weight_h_exp(1,:))>1
                for j = 2:length(weight_h_exp(1,:))
                    w_eight=[w_eight,zeros(1,L-1),weight_constraint(j)];
                end
            end
        end
    else
        w_eight=zeros(1,L*length(weight_h_exp(1,:)));
    end
    % Here we implement the matrix which links central-deviance parameters and original parameters
    % The i2 restriction is not implemented (correctly) yet!!!
    if i2
      if i1
        % First and second order restrictions
        des_mat=zeros((L-2)*length(weight_h_exp(1,:)),L*length(weight_h_exp(1,:)));
        for i = 1:L-2
          des_mat(i,i+(0:(length(weight_h_exp(1,:))-1))*L)=1;
          des_mat(i,L-1+(0:(length(weight_h_exp(1,:))-1))*L)=-(L-i); 
          des_mat(i,L+(0:(length(weight_h_exp(1,:))-1))*L)=(L-i-1);
        end
        % new 08.02.2012: the if loop is new
        if length(weight_h_exp(1,:))>1
          for j = 1:max(1,(length(weight_h_exp(1,:))-1))
            for i = 1:L-2               %reg_mat[600,600
              des_mat(i+j*(L-2),i)=-1;
              des_mat(i+j*(L-2),i+j*L)=1;
              des_mat(i+j*(L-2),L-1)=L-i;
              des_mat(i+j*(L-2),L)=-(L-i-1);
              des_mat(i+j*(L-2),L-1+j*L)=-(L-i);
              des_mat(i+j*(L-2),L+j*L)=L-i-1;
            end
          end
        end
      else
        des_mat=zeros((L-1)*length(weight_h_exp(1,:)),L*length(weight_h_exp(1,:)));
        for i = 1:L-1
          des_mat(i,i+(0:(length(weight_h_exp(1,:))-1))*L)=1;
          des_mat(i,L+(0:(length(weight_h_exp(1,:))-1))*L)=-(i-1)/(L-1);
        end
        % new 08.02.2012: the if loop is new/modified
        if length(weight_h_exp(1,:))>1
          for j = 1:max(1,(length(weight_h_exp(1,:))-1))
            for i = 1:L-1
              des_mat(i+j*(L-1),i)=-1;
              des_mat(i+j*(L-1),i+j*L)=1;
              des_mat(i+j*(L-1),L)=(i-1)/(L-1);
              des_mat(i+j*(L-1),L+j*L)=-(i-1)/(L-1);
            end
          end
        end
      end
    else
        if i1    %lambda_cross=0
            des_mat=zeros((L-1)*length(weight_h_exp(1,:)),L*length(weight_h_exp(1,:)));
            % The cross-sectional regularization can be directly implemented into reg_mat (it adresses the central deviance parameters)
            for i = 1:L-1  %i<-1      
                % The i1-constraint is imposed on b_max(0,Lag) (instead of b_L as in original procedure) in order to avoid a conflict with the exponential decay requirement
                if Lag<1
                    des_mat(i,i+1+(0:(length(weight_h_exp(1,:))-1))*L)=1;
                    des_mat(i,1+(0:(length(weight_h_exp(1,:))-1))*L)=-1;
                else
                    % Lag cannot be larger than (L-1)/2 (symmetric filter)
                    if i<(Lag+1)
                        jjj=i;
                    else
                        jjj=i+1;
                    end
                    des_mat(i,jjj+(0:(length(weight_h_exp(1,:))-1))*L)=1;
                    des_mat(i,Lag+1+(0:(length(weight_h_exp(1,:))-1))*L)=-1;
                end
            end
            % new 08.02.2012: the if loop is new/modified
            if length(weight_h_exp(1,:))>1
                for j = 1:max(1,(length(weight_h_exp(1,:))-1))   %j<-1
                    for i = 1:L-1       
                        % The i1-constraint is imposed on b_max(0,Lag) (instead of b_L as in original procedure)
                        % in order to avoid a conflict with the exponential decay requirement
                        if Lag<1           
                            des_mat(i+j*(L-1),i+1)=-1;
                            des_mat(i+j*(L-1),1)=1;
                            des_mat(i+j*(L-1),i+1+j*L)=1;
                            des_mat(i+j*(L-1),1+j*L)=-1;
                        else           
                            % Lag cannot be larger than (L-1)/2 (symmetric filter)
                            if i<(Lag+1)
                                jjj=i;
                            else
                                jjj=i+1;
                            end
                            des_mat(i+j*(L-1),jjj)=-1;
                            des_mat(i+j*(L-1),Lag+1)=1;
                            des_mat(i+j*(L-1),jjj+j*L)=1;
                            des_mat(i+j*(L-1),Lag+1+j*L)=-1;
                        end
                        % The cross sectional regularization is implemented directly into 
                        % reg_mat (it addresses central deviance parameters!)
                        %    reg_mat[i+(j)*(L-1),i+(j)*(L-1)]<-lambda_cross
                    end
                end             % det(reg_mat)         lambda_cross<-0.3
            end
        else
            des_mat=zeros(L*length(weight_h_exp(1,:)),L*length(weight_h_exp(1,:)));
                for i = 1:L
                    des_mat(i,i+(0:(length(weight_h_exp(1,:))-1))*L)=1;
                end
                % new 08.02.2012: the if loop is new/modified
                if length(weight_h_exp(1,:))>1
                    for j = 1:max(1,(length(weight_h_exp(1,:))-1))
                        for i = 1:L
                            des_mat(i+j*L,i)=-1;
                            des_mat(i+j*L,i+j*L)=1;
                        end
                    end
                end
        end
    end
    % Here we fold all three regularizations (cross, smooth and decay) into a single reg-matrix
    % The smoothness and decay terms address original parameters and must be transformed (by des_mat) in order to
    % conform to the central-deviance parameterization as well as to first (and/or second) order constraints
    % begin new 08.02.2012: the if-loop is new    
    if length(weight_h_exp(1,:))>1
        reg_t=Q_smooth+Q_decay+Q_centraldev_original'*Q_cross*Q_centraldev_original;
    else
        reg_t=Q_smooth+Q_decay;
    end
    reg_mat=des_mat*reg_t*des_mat';
    reg_xtxy=des_mat*reg_t*w_eight';     %+t(w_eight)%*%reg_t%*%t(des_mat)
    % end new 08.02.2012
    
