function [des_mat,reg_mat,reg_xtxy,w_eight]=mat_func2(i1,i2,L,weight_h_exp,lambda_decay,lambda_cross,lambda_smooth,Lag,weight_constraint,shift_constraint)
    %Ginters Buss 2012
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % if useful, cite Ginters Buss "Real-time signal extraction with
    % regularized multivariate direct filter approach",
    % in Journal of Forecasting
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % updated to Wildi I-mdfa_new10.r (2012 aug 2); a few bugs corrected
    % translated from Wildi I-mdfa_new6.r code 
    % New 2012-code: Computes regularization matrices and expresses 
    % parameters in central-deviance form
    % [des_mat,reg_mat,reg_xtxy,w_eight]=mat_func(i1,i2,L,weight_h_exp,
    % lambda_decay,lambda_cross,lambda_smooth,Lag,weight_constraint)
    % Lag cut suspended by GB 2012
    %if Lag>(L-1)/2
    %    disp('Lag larger than L/2!!!!! Will be trimmed automtically to L/2 (symmetric filter)');
    %    Lag=fix(L/2);
    %end
    lambda_smooth=abs(lambda_smooth);
    lambda_cross=abs(lambda_cross);
    lambda_decay=abs(lambda_decay);
    % The smoothness and decay regularization are conveniently (rightly) 
    % implemented on original parameters
    % The Q_smooth and Q_decay matrices address regularizations for original
    % unconstrained parameters (Therefore dimension L^2)
    % At the end, the matrix des_mat is used to map these regularizations 
    % to central-deviance parameters
    % accounting for first order constraints!!!!
    Q_smooth=zeros(L*length(weight_h_exp(1,:)),L*length(weight_h_exp(1,:)));
    Q_decay=Q_smooth;
    % Cross-sectional regularization if dimension>1
    if length(weight_h_exp(1,:))>1
        % The cross-sectional regularization is conveniently implemented 
        % on central-deviance parameters. The regularization is expressed on the
        % unconstrained central-deviance parameters (dimension L), then 
        % mapped to the original (unconstrained) parameters (dimension L) with Q_centraldev_original
        % and then mapped back to central-deviance with constraint (dim L-1) 
        % with des_mat (mathematically unnecessarily complicate but more convenient to implement in code).
        Q_cross=Q_decay;
        Q_centraldev_original=Q_cross;
    end
    for i=1:L
        % For symmetric filters or any historical filter with Lag>0 the 
        % decay must be symmetric about b_max(0,Lag) zunehmen
        Q_decay(i,i)=lambda_decay*(1+lambda_decay)^(2*abs(i-1-max(0,Lag)));
        if i==1
            Q_smooth(i,i:i+2)=lambda_smooth*[1,-2,1];
        elseif i==2
            Q_smooth(i,i-1:i+2)=lambda_smooth*[-2,5,-4,1];
        elseif i==L 
            Q_smooth(i,i-2:i)=lambda_smooth*[1,-2,1];
        elseif i==L-1
            Q_smooth(i,i-2:i+1)=lambda_smooth*[1,-4,5,-2];
        else
            Q_smooth(i,i-2:i+2)=lambda_smooth*[1,-4,6,-4,1];
        end
    end
    
    if length(weight_h_exp(1,:))>1
        for j=1:max(1,length(weight_h_exp(1,:))-1)   %j<-1
            Q_smooth(j*L+(1:L),j*L+(1:L))=Q_smooth(1:L,1:L);
            Q_decay(j*L+(1:L),j*L+(1:L))=Q_decay(1:L,1:L);
        end
        Q_centraldev_original(1:L,1:L)=diag(ones(1,L));
        Q_centraldev_original(1:L,L+(1:L))=(-1)*diag(ones(1,L));
        for i=2:length(weight_h_exp(1,:))
            Q_centraldev_original((i-1)*L+(1:L),1:L)=diag(ones(1,L));
            Q_centraldev_original((i-1)*L+(1:L),(i-1)*L+(1:L))=diag(ones(1,L));
            Q_centraldev_original(1:L,(i-1)*L+(1:L))=(-1)*diag(ones(1,L));
        end
        Q_centraldev_original=inv(Q_centraldev_original);
        %Q_cross(L+(1:((length(weight_h_exp(1,:))-1)*L)),L+(1:((length(weight_h_exp(1,:))-1)*L)))=lambda_cross*diag(ones(1,((length(weight_h_exp(1,:))-1)*L)));
        % 30.07.2012:new definition (parametrization) of Q_cross (Lambda_{cross} in the elements-paper)
        Q_cross=eye(size(Q_cross,1));
        for i=1:length(weight_h_exp(1,:))
            for j=1:L
                Q_cross((i-1)*L+j,j+(0:(length(weight_h_exp(1,:))-1))*L)=Q_cross((i-1)*L+j,j+(0:(length(weight_h_exp(1,:))-1))*L)-1/length(weight_h_exp(1,:));
            end
        end
        Q_cross=Q_cross*lambda_cross;    
     
    end
    % new 21.06.2012: w_eight vector
    % weight vector (relevant if i1<-T)
    if i1
        if i2
            % Modifications 17.04.2012 : the definition of the vector w_eight has been generalized:
            % The new definition allows to impose constraints to b_Lag, b_{Lag+1} instead of b_{L-1} and b_L
            % Therefore the decay regularization does not potentially conflict with filter constraints
            if Lag<1
                 w_eight=[-(Lag-1)*weight_constraint(1)-shift_constraint(1),Lag*weight_constraint(1)+shift_constraint(1),zeros(1,L-2)];
            else
                 w_eight=[zeros(1,Lag),weight_constraint(1)-shift_constraint(1),shift_constraint(1),zeros(1,L-Lag-2)];
            end
            %w_eight=[zeros(1,L-2),(L-1)*weight_constraint(1),-(L-2)*weight_constraint(1)];
            if length(weight_h_exp(1,:))>1
                for j = 2:length(weight_h_exp(1,:))
                    if Lag<1
                        w_eight=[w_eight,-(Lag-1)*weight_constraint(j)-shift_constraint(j),Lag*weight_constraint(j)+shift_constraint(j),zeros(1,L-2)];
                    else
                        w_eight=[w_eight,[zeros(1,Lag),weight_constraint(j)-shift_constraint(j),shift_constraint(j),zeros(1,L-Lag-2)]];
                    end
                    %w_eight=[w_eight,zeros(1,L-2),(L-1)*weight_constraint(j),-(L-2)*weight_constraint(j)];
                end
            end
        else
            if Lag<1
            %w_eight=[zeros(1,L-1),weight_constraint(1)];
                w_eight=[weight_constraint(1),zeros(1,L-1)];
            else
                w_eight=[zeros(1,Lag),weight_constraint(1),zeros(1,L-Lag-1)];
            end
            if length(weight_h_exp(1,:))>1
                for j = 2:length(weight_h_exp(1,:))
                    %w_eight=[w_eight,zeros(1,L-1),weight_constraint(j)];
                    if Lag<1
                        w_eight=[w_eight,weight_constraint(j),zeros(1,L-1)];
                    else
                        w_eight=[w_eight,zeros(1,Lag),weight_constraint(j),zeros(1,L-Lag-1)];
                    end
                end
            end
        end
    else
        if i2
            if Lag<1
                w_eight=[0,shift_constraint(1)/(1-Lag),zeros(1,L-2)];
            else
                w_eight=[zeros(1,Lag+1),shift_constraint(1),zeros(1,L-Lag-2)];
            end
            %w_eight=zeros(1,L*length(weight_h_exp(1,:)));
            if length(weight_h_exp(1,:))>1
                for j=2:length(weight_h_exp(1,:))
                    if Lag<1
                        w_eight=[w_eight,[0,shift_constraint(j)/(1-Lag),zeros(1,L-2)]];
                    else
                        w_eight=[w_eight,[zeros(1,Lag+1),shift_constraint(j),zeros(1,L-Lag-2)]];
                    end
                end
            end
        else
            w_eight=zeros(1,L*length(weight_h_exp(1,:)));
        end
    end
    % Here we implement the matrix which links freely determined central-deviance parameters and constrained original parameters
    % In my `elements'-paper t(des_mat) corresponds to A%*%R (R links constrained and unconstrained parameters and A maps central-deviance to original parameters)
    %   Please note that:
    %   1. Here I'm working with central-deviance parameters (in the paper I'm working with original parameters)
    %   2. The same matrix R applies to either parameter set
    %   3. If I work with central-deviance parameters then R maps the freely determined set to the constrained (central-deviance)
    %       and A then maps the constrained (central-deviance) set to original constrained parameters.
    % Modifications 17.04.2012: we generalize the definition of des_mat in the case of i2<-T (i2<-F is OK)

    if i2
      if i1
        % First and second order restrictions
        des_mat=zeros((L-2)*length(weight_h_exp(1,:)),L*length(weight_h_exp(1,:)));
        
        for i = 1:L-2
            if Lag<1
                des_mat(i,i+2+(0:(length(weight_h_exp(1,:))-1))*L)=1;
                des_mat(i,1+(0:(length(weight_h_exp(1,:))-1))*L)=i;
                des_mat(i,2+(0:(length(weight_h_exp(1,:))-1))*L)=-(i+1);
            else
                if i<(Lag+1)
                    jjj=i;
                    jjj2=-(Lag+2-i);
                    jjj3=Lag+1-i;
                else
                    jjj=i+2;
                    jjj2=i-Lag;
                    jjj3=-(i-Lag+1);
                end
                des_mat(i,jjj+(0:(length(weight_h_exp(1,:))-1))*L)=1;
                des_mat(i,Lag+1+(0:(length(weight_h_exp(1,:))-1))*L)=jjj2;
                des_mat(i,Lag+2+(0:(length(weight_h_exp(1,:))-1))*L)=jjj3;
            end
        end
        if length(weight_h_exp(1,:))>1
          for j = 1:max(1,(length(weight_h_exp(1,:))-1))
            for i = 1:L-2
                if Lag<1
                    des_mat(i+j*(L-2),i+2)=-1;
                    des_mat(i+j*(L-2),1)=-i;
                    des_mat(i+j*(L-2),2)=(i+1);
                else
                    if i<(Lag+1)
                        jjj=i;
                        jjj2=-(Lag+2-i);
                        jjj3=Lag+1-i;
                    else
                        jjj=i+2;
                        jjj2=i-Lag;
                        jjj3=-(i-Lag+1);
                    end
                    des_mat(i+j*(L-2),jjj)=-1;
                    des_mat(i+j*(L-2),Lag+1)=-jjj2;
                    des_mat(i+j*(L-2),Lag+2)=-jjj3;
                end
                if Lag<1
                    des_mat(i+j*(L-2),i+2+j*L)=1;
                    des_mat(i+j*(L-2),1+j*L)=i;
                    des_mat(i+j*(L-2),2+j*L)=-(i+1);
                else
                    if i<(Lag+1)
                        jjj=i+j*L;
                        jjj2=-(Lag+2-i);
                        jjj3=Lag+1-i;
                    else
                        jjj=i+2+j*L;
                        jjj2=i-Lag;
                        jjj3=-(i-Lag+1);
                    end
                    des_mat(i+j*(L-2),jjj)=1;
                    des_mat(i+j*(L-2),Lag+1+j*L)=jjj2;
                    des_mat(i+j*(L-2),Lag+2+j*L)=jjj3;
                end
            end
          end
        end
      else
          % Modifications 17.04.2012: we generalize the definition of des_mat in the case of i2<-T (i2<-F is OK)
          % new 21.06.2012: new definition of des_mat in the case i1<-F and i2<-T
          des_mat=zeros((L-1)*length(weight_h_exp(1,:)),L*length(weight_h_exp(1,:)));
        for i = 1:L-1
            if Lag<1
                % For Lag<1 we have to impose the restriction to b2 (b1 does not appear in the i2<-T and i1<-F case!)
                if i<2
                    jjj=i;
                else
                    jjj=i+1;
                end
                des_mat(i,jjj+(0:(length(weight_h_exp(1,:))-1))*L)=1;
                if i==1
                    jjj1=Lag;
                else
                    jjj1=Lag-i;
                end
                des_mat(i,1+1+(0:(length(weight_h_exp(1,:))-1))*L)=jjj1/(1-Lag);
            else
                if i<(Lag+2)
                    jjj=i;
                    jjj2=Lag+1-i;
                else
                    jjj=i+1;
                    jjj2=Lag-i;
                end
               
                des_mat(i,jjj+(0:(length(weight_h_exp(1,:))-1))*L)=1;
                des_mat(i,Lag+2+(0:(length(weight_h_exp(1,:))-1))*L)=jjj2;
            end
        end
        % new 08.02.2012: the if loop is new/modified
        if length(weight_h_exp(1,:))>1
          for j = 1:max(1,(length(weight_h_exp(1,:))-1))
            for i = 1:L-1
                if Lag<1
                    % For Lag<1 we have to impose the restriction to b2 (b1 does not appear in the i2<-T and i1<-F case!)
                    if i<2
                        jjj=i;
                    else
                        jjj=i+1;
                    end
                    des_mat(i+j*(L-1),jjj)=-1;
                    if i==1
                        jjj2=Lag;
                    else
                        jjj2=Lag-i;
                    end
                    des_mat(i+j*(L-1),1+1)=-jjj2/(1-Lag);
                else
                    if i<(Lag+2)
                        jjj=i;
                        jjj2=Lag+1-i;
                    else
                        jjj=i+1;
                        jjj2=Lag-i;
                    end
                    des_mat(i+j*(L-1),jjj)=-1;
                    des_mat(i+j*(L-1),Lag+2)=-jjj2;
                end
                if Lag<1
                    % For Lag<1 we have to impose the restriction to b2 (b1 does not appear in the i2<-T and i1<-F case!)
                    if i<2
                        jjj=i;
                    else
                        jjj=i+1;
                    end
                    des_mat(i+j*(L-1),jjj+j*L)=1;
                    if i==1
                        jjj2=Lag;
                    else
                        jjj2=Lag-i;
                    end
                    des_mat(i+j*(L-1),1+1+j*L)=jjj2/(1-Lag);
                else
                    if i<(Lag+2)
                        jjj=i;
                        jjj2=Lag+1-i;
                    else
                        jjj=i+1;
                        jjj2=Lag-i;
                    end
                    des_mat(i+j*(L-1),jjj+j*L)=1;
                    des_mat(i+j*(L-1),Lag+2+j*L)=jjj2;
                end
            end
          end
        end
      end
    else
        if i1    %lambda_cross=0
            des_mat=zeros((L-1)*length(weight_h_exp(1,:)),L*length(weight_h_exp(1,:)));
            % The cross-sectional regularization can be directly implemented into reg_mat (it adresses the central deviance parameters)
            for i = 1:L-1  %i<-1      
                % The i1-constraint is imposed on b_max(0,Lag) (instead of b_L as in original procedure) in order to avoid a conflict with the exponential decay requirement
                if Lag<1
                    des_mat(i,i+1+(0:(length(weight_h_exp(1,:))-1))*L)=1;
                    des_mat(i,1+(0:(length(weight_h_exp(1,:))-1))*L)=-1;
                else
                    % Lag cannot be larger than (L-1)/2 (symmetric filter)
                    if i<(Lag+1)
                        jjj=i;
                    else
                        jjj=i+1;
                    end
                    des_mat(i,jjj+(0:(length(weight_h_exp(1,:))-1))*L)=1;
                    des_mat(i,Lag+1+(0:(length(weight_h_exp(1,:))-1))*L)=-1;
                end
            end
            % new 08.02.2012: the if loop is new/modified
            if length(weight_h_exp(1,:))>1
                for j = 1:max(1,(length(weight_h_exp(1,:))-1))   %j<-1
                    for i = 1:L-1       
                        % The i1-constraint is imposed on b_max(0,Lag) (instead of b_L as in original procedure)
                        % in order to avoid a conflict with the exponential decay requirement
                        if Lag<1           
                            des_mat(i+j*(L-1),i+1)=-1;
                            des_mat(i+j*(L-1),1)=1;
                            des_mat(i+j*(L-1),i+1+j*L)=1;
                            des_mat(i+j*(L-1),1+j*L)=-1;
                        else           
                            % Lag cannot be larger than (L-1)/2 (symmetric filter)
                            if i<(Lag+1)
                                jjj=i;
                            else
                                jjj=i+1;
                            end
                            des_mat(i+j*(L-1),jjj)=-1;
                            des_mat(i+j*(L-1),Lag+1)=1;
                            des_mat(i+j*(L-1),jjj+j*L)=1;
                            des_mat(i+j*(L-1),Lag+1+j*L)=-1;
                        end
                        % The cross sectional regularization is implemented directly into 
                        % reg_mat (it addresses central deviance parameters!)
                        %    reg_mat[i+(j)*(L-1),i+(j)*(L-1)]<-lambda_cross
                    end
                end             % det(reg_mat)         lambda_cross<-0.3
            end
        else
            des_mat=zeros(L*length(weight_h_exp(1,:)),L*length(weight_h_exp(1,:)));
                for i = 1:L
                    des_mat(i,i+(0:(length(weight_h_exp(1,:))-1))*L)=1;
                end
                % new 08.02.2012: the if loop is new/modified
                if length(weight_h_exp(1,:))>1
                    for j = 1:max(1,(length(weight_h_exp(1,:))-1))
                        for i = 1:L
                            des_mat(i+j*L,i)=-1;
                            des_mat(i+j*L,i+j*L)=1;
                        end
                    end
                end
        end
    end
    % 30.07.2012: patch which does apply A^{-1} to A*R giving R where A and R are defined in my elememts paper
    % and des_mat=A*R i.e.  t(Q_centraldev_original%*%t(des_mat)) is R (called des_mat in my code)
    des_mat=(Q_centraldev_original*des_mat')';
    % Here we fold all three regularizations (cross, smooth and decay) into a single reg-matrix
    % The smoothness and decay terms address original parameters and must be transformed (by des_mat) in order to
    % conform to the central-deviance parameterization as well as to first (and/or second) order constraints
    % begin new 08.02.2012: the if-loop is new    
    if length(weight_h_exp(1,:))>1
        %reg_t=Q_smooth+Q_decay+Q_centraldev_original'*Q_cross*Q_centraldev_original;
        % 30.07.2012 The cross-sectional regularization term is now simpler (without A-transformation)
        reg_t=Q_smooth+Q_decay+Q_cross;
    else
        reg_t=Q_smooth+Q_decay;
    end
    reg_mat=des_mat*reg_t*des_mat';
    reg_xtxy=des_mat*reg_t*w_eight';     %+t(w_eight)%*%reg_t%*%t(des_mat)
    % end new 08.02.2012
    
