function [b,trffkt,rever,degrees_freedom,aic,freezed_degrees]=mdfa_analytic_new(K,L,lambda,weight_func,Lag,Gamma,expweight,cutoff,i1,i2,weight_constraint,lambda_cross,lambda_decay,lambda_smooth)
    %Ginters Buss 2012
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % if useful, cite Ginters Buss "Real-time signal extraction with
    % regularized multivariate direct filter approach",
    % in Journal of Forecasting
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % translated from Wildi I-mdfa_new6.r code 
    % New 2012 I-MDFA code: might crash if dimension is 1 (univariate setting): I did not test this!!!
    % Generalizes old mdfa code when regularization parameters are set to zero
    % This new function can deal with very richly parametrized designs 
    % (high-dimensional multivariate with long lag structures)
    % Regularization affect/control smoothness, rate of decay and cross-sectional 
    % similarity of filter parameters/weights

    % In order to enhance numerical speed this call could be done outside (as long as L and Lag are fixed)
    Gamma=reshape(Gamma,1,[]);
    spec_mat=spec_mat_comp(weight_func,L,Lag);
    % weighting of amplitude function in stopband
    sizewf=size(weight_func);
    omega_Gamma=fix(cutoff*K/pi);
    if (K-omega_Gamma+1)>0
        expweight_vec=[ones(1,omega_Gamma),(1:(K-omega_Gamma+1)).^(expweight/2)];
        weight_h=weight_func.*(expweight_vec'*ones(1,sizewf(2)));
    else
        expweight_vec=ones(1,K+1);
        weight_h=weight_func.*(expweight_vec'*ones(1,sizewf(2)));
    end
    % Frequency zero receives half weight
    weight_h(1,:)=weight_h(1,:)*0.5;
    % DFT target variable
    weight_target=weight_h(:,1);
    % Rotate all DFT's such that weight_target is real (rotation does not alter mean-square error)
    weight_h=weight_h.*(exp(-1i*angle(weight_target))*ones(1,sizewf(2)));
    weight_target=real(weight_target.*exp(-1i*angle(weight_target)));
    % DFT's explaining variables: target variable can be an explaining variable too
    weight_h_exp=weight_h(:,2:end);
    % The spectral matrix is inflated in stopband: effect of expweight
    sizesm=size(spec_mat);
    spec_mat=spec_mat.*(ones(sizesm(1),1)*expweight_vec);
    % Compute design matrix and regularization matrix
    [des_mat,reg_mat,reg_xtxy,w_eight]=mat_func(i1,i2,L,weight_h_exp,lambda_decay,lambda_cross,lambda_smooth,Lag,weight_constraint);
    % Solve estimation problem in space of b_main and b_dev
    mat_x=des_mat*spec_mat;
    sizemx=size(mat_x);
    %X_new=real(mat_x)'+(ones(sizemx(1),1)*(sqrt(1+Gamma*lambda)*1i))'.*imag(mat_x)';
    X_new=(real(mat_x)+imag(mat_x).*(ones(sizemx(1),1)*(sqrt(1+Gamma*lambda)*1i)))';
    % xtx can be written either in Conj(X_new)%*%X_new or as below:
    xtx=real(X_new)'*real(X_new)+imag(X_new)'*imag(X_new);
    % The filter restrictions (i1<-T) appear as constants on the right hand-side of the equation:
    xtxy=(real(w_eight*spec_mat)*real(spec_mat'*des_mat')+ ...
        imag(w_eight*(spec_mat.*(ones(sizesm(1),1)*sqrt(1+Gamma*lambda))))* ...
        (-1)*imag((spec_mat.*(ones(sizesm(1),1)*sqrt(1+Gamma*lambda)))'*des_mat'))';
    % scaler makes scales of regularization and unconstrained optimization `similar'
    scaler=mean(diag(xtx));
    %X_inv=inv(xtx+scaler*reg_mat);
    bh=(xtx+scaler*reg_mat)\(((real(X_new).*(weight_target*ones(1,sizemx(1))))'*Gamma')-xtxy-scaler*reg_xtxy);
    % the last two filter weights are functions of the previous ones 
    % through the first and second order restrictions
    b=nan(L,length(weight_h_exp(1,:)));
    % Reconstruct original parameters
    bhh=des_mat'*bh;
    for k = 1:L 
        b(k,:)=bhh(k+(0:(length(weight_h_exp(1,:))-1))*L);
    end
    % If i1 is imposed then one has to correct final parameters for imposed constraints
    if i1
        if i2
            b(L-1,:)=b(L-1,:)+(L-1)*weight_constraint;
            b(L,:)=b(L,:)-(L-2)*weight_constraint;  
        else
            b(L,:)=b(L,:)+weight_constraint;
        end
    end
    % The target Y in the frequency-domain is the real vector weight_target*Gamma (both vectors are real)
    % The Regression estimate (the smoother) of Y is the following expression:
    %trt=reshape((Gamma.*weight_target')*(X_new*((xtx+scaler*reg_mat)\real(X_new)'))',[],1);
    % This expression is identical to trt computed below if lambda=0 (otherwise trth is identical to Re(trt)+1.i*sqrt(1+lambda*Gamma)*Im(trt))
    % The projection matrix is therefore:
    Proj_mat=X_new*((xtx+scaler*reg_mat)\real(X_new)');
    % The residual projection matrix is
    res_mat=diag(ones(1,sizemx(2)))-Proj_mat;
    % DFA criterion: first possibility (all three variants are identical)
    %  sum(abs(res_mat%*%(weight_target*Gamma))^2)
    % Residuals
    %resi=res_mat*(weight_target.*Gamma');
    % DFA criterion: second possibility
    % t(Conj(resi))*resi
    % t((weight_target*Gamma))%*%(t(Conj(res_mat))%*%(res_mat))%*%(weight_target*Gamma)
    %  sum(diag(res_mat))
    %  sum(diag(t(res_mat%*%Conj(res_mat))))

    % The interesting `effective degrees of freedom' used here emphasizes 
    % an unbiased estimate of the mean-square residual
    % See  http://en.wikipedia.org/wiki/Degrees_of_freedom_(statistics) 
    % (effective degrees of freedom: the expression tr((I-H)(I-H)')
    % Note that res_mat=I-H see above
    % Then (Y-Y^)(Y-Y^)/tr((I-H)(I-H)') is an unbiased estimate of the 
    % mean-square residual error (in our case Y=weight_target*Gamma. see above)
    % This correcting `effective degrees of freedom' can then be 
    % used to implement a generalized AIC, see below
    % Neither of the other proposed definitions of `effective degrees of freedom' 
    % on Wiki-site tackle this problem (in particular the trace of 
    % the smoothing operator is not what we want!!!!)
    degrees_freedom=2*real(sum(diag((res_mat)'*res_mat)))-1;
    freezed_degrees=2*K+1-degrees_freedom;
    % degrees_freedom<-K-Re(sum(diag(Proj_mat)))
    % DFA Criterion: third possibility (here an additional normalization by 1/(2*(K+1)^2))
    %rever=sum(abs(Gamma'.*weight_target-trt).^2)/(2*(K+1)^2); 
    %rever=sum(abs(Gamma'.*weight_target-real(trt)-1i*sqrt(1+lambda*Gamma)'.*imag(trt)).^2)/(2*(K+1).^2);
    % from mdfa old: rever=sum(abs(Gamma_mdfa.*weight_target-real(trt)-1i*sqrt(1+lambda_mdfa*Gamma_mdfa).*imag(trt)).^2)/(2*(K_mdfa+1).^2);
    % Transferfunction
    trffkt=nan(K+1,length(weight_h_exp(1,:)));
    %trffkth=trffkt;
    trffkt(1,:)=sum(b);
    %trffkth(1,:)=trffkt(1,:);
    %  b<-scale(b,center=T,scale=F)
    for j = 1:length(weight_h_exp(1,:))
        for k = 0:K   
            trffkt(k+1,j)=exp(1i*k*(0:(L-1))*pi/(K))*b(:,j);
        end
    end
    trt=sum(((trffkt.*(reshape(exp(1i*(0-Lag)*pi*(0:K)/K),[],1)*ones(1,sizewf(2)-1))).*weight_h_exp),2);
    rever=sum(abs(Gamma'.*weight_target-real(trt)-1i*sqrt(1+lambda*Gamma').*imag(trt)).^2)/(2*(K+1)^2);
    %aic<-ifelse(degrees_freedom<K+1&degrees_freedom>1,log(rever)+2*(K-degrees_freedom)/(K)+2*(K-degrees_freedom)*(K-degrees_freedom+1)/(K*(degrees_freedom-1)),NA)
    if (degrees_freedom<(K+1) && degrees_freedom>1)
        aic=log(rever)+2*(K-degrees_freedom+1)/(degrees_freedom-2);
    else
        aic=nan;
    end
    %aic=ifelse(degrees_freedom<K+1&degrees_freedom>1,log(rever)+2*(K-degrees_freedom+1)/(degrees_freedom-2),NA)
    % ts.plot(abs(trffkt))   exp(1.i*(0-Lag)*pi*(0:(K))/K)*

    
